
package JavaBasicsII;
import pi1.*;


/*************************************************************************
 *
 *  Usage:
 *  	Java PlayThatTune         # play pitch 5 for duration 1.4 seconds
 *  	5 1.4
 *  
 *  Usage:
 *  	java PlayThatTune < PlayThatTune-Elise.txt
 *
 *************************************************************************/

public class PlayThatTune {

    // take weighted sum of two arrays
    public static double[] sum(double[] a, double[] b, double awt, double bwt) {

        // precondition: arrays have the same length
        assert(a.length == b.length);

        // compute the weighted sum
        double[] c = new double[a.length];
        for (int i = 0; i < a.length; i++) {
            c[i] = a[i]*awt + b[i]*bwt;
        }
        return c;
    } 

    // create a pure tone of the given frequency for the given duration
    public static double[] tone(double hz, double duration) { 
        int SAMPLES_PER_SECOND = 64000;
        int N = (int) (SAMPLES_PER_SECOND * duration);
        double[] a = new double[N+1];
        for (int i = 0; i <= N; i++) {
            a[i] = Math.sin(2 * Math.PI * i * hz / SAMPLES_PER_SECOND);
        }
        return a; 
    } 

    // create a note with harmonics of of the given pitch (0 = middle C)
    public static double[] note(int pitch, double t) {
        double hz = 440.0 * Math.pow(2, pitch / 12.0);
        double[] a  = tone(hz, t);
        double[] hi = tone(2*hz, t);
        double[] lo = tone(hz/2, t);
        double[] h  = sum(hi, lo, .5, .5);
        return sum(a, h, .5, .5);
    }


    // read in notes from standard input and play them on standard audio
    public static void main(String[] args) {
    	
        // read in pitch-duration pairs from standard input
        while (!StdIn.isEmpty()) {
            int pitch = StdIn.readInt();
            double duration = StdIn.readDouble();
            double[] a = note(pitch, duration);
            
            StdOut.print(pitch + " ");
            if (duration>=0.75 || pitch<-50) StdOut.println("");
            
            StdAudio.play(a);
        }

       // needed to terminate program - known Java bug
       System.exit(0);
    } 
}
